'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _vinyl = require('vinyl');

var _vinyl2 = _interopRequireDefault(_vinyl);

var _minimatch = require('minimatch');

var _minimatch2 = _interopRequireDefault(_minimatch);

var _cssMediaquery = require('css-mediaquery');

var _cssMediaquery2 = _interopRequireDefault(_cssMediaquery);

var _sharp = require('sharp');

var _sharp2 = _interopRequireDefault(_sharp);

var _imagemin = require('imagemin');

var _imagemin2 = _interopRequireDefault(_imagemin);

var _imageminWebp = require('imagemin-webp');

var _imageminWebp2 = _interopRequireDefault(_imageminWebp);

var _imageminMozjpeg = require('imagemin-mozjpeg');

var _imageminMozjpeg2 = _interopRequireDefault(_imageminMozjpeg);

var _imageminZopfli = require('imagemin-zopfli');

var _imageminZopfli2 = _interopRequireDefault(_imageminZopfli);

var _imageminGiflossy = require('imagemin-giflossy');

var _imageminGiflossy2 = _interopRequireDefault(_imageminGiflossy);

var _imageminSvgo = require('imagemin-svgo');

var _imageminSvgo2 = _interopRequireDefault(_imageminSvgo);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var extensions = {
	webp: /^webp$/,
	jpg: /^jp(e|)g$/,
	png: /^png$/,
	gif: /^gif$/,
	svg: /^svg$/
};

var isMediaQuery = /^\s*(\(\s*((max|min)-|)(width|height)\s*:\s*\d+\w*\s*\)\s*(,|and)\s*)*\(\s*((max|min)-|)(width|height)\s*:\s*\d+\w*\s*\)\s*$/g;

var SrcsetGenerator = function () {
	_createClass(SrcsetGenerator, null, [{
		key: 'TypeIsSupported',


		/**
   * Check image type
   * @param  {String} type
   * @return {Boolean}
   */
		value: function TypeIsSupported(type) {
			return SrcsetGenerator.ExtensionsList.some(function (_) {
				return _.test(type);
			});
		}
	}]);

	function SrcsetGenerator() {
		var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

		_classCallCheck(this, SrcsetGenerator);

		this.processing = {
			webp: {
				quality: 100
			},
			jpg: {
				quality: 100
			},
			png: {}
		};
		this.optimization = {
			webp: (0, _imageminWebp2.default)({
				quality: 100
			}),
			jpg: (0, _imageminMozjpeg2.default)({
				quality: 100
			}),
			png: (0, _imageminZopfli2.default)(),
			gif: (0, _imageminGiflossy2.default)(),
			svg: (0, _imageminSvgo2.default)()
		};

		this.postfix = function (width, mul) {
			return mul === 1 ? '' : '@' + width + 'w';
		};

		this.skipOptimization = false;


		if ((typeof config === 'undefined' ? 'undefined' : _typeof(config)) == 'object') {
			var processing = config.processing,
			    optimization = config.optimization,
			    postfix = config.postfix,
			    skipOptimization = config.skipOptimization;


			Object.assign(this.processing, processing);
			Object.assign(this.optimization, optimization);

			if (typeof postfix == 'function') {
				this.postfix = postfix;
			}

			if (typeof skipOptimization == 'boolean') {
				this.skipOptimization = skipOptimization;
			}
		}
	}

	/**
  * Create set of sources form original image.
  * @param  {Vinyl}    source
  * @param  {Object}   config
  * @param  {Function} push
  * @return {Promise<Array<Vinyl>>}
  */


	_createClass(SrcsetGenerator, [{
		key: 'generate',
		value: function generate(source) {
			var _this = this;

			var _config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

			var push = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;


			if (!_vinyl2.default.isVinyl(source) || source.isNull() || source.isStream()) {
				throw new Error('Invalid source.');
			}

			var config = Object.assign({
				format: [],
				width: [],
				postfix: false,
				processing: false,
				optimization: false,
				skipOptimization: this.skipOptimization
			}, _config);

			var TypeIsSupported = SrcsetGenerator.TypeIsSupported,
			    Extensions = SrcsetGenerator.Extensions,
			    sourceType = source.extname.replace(/^\./, '');


			if (!TypeIsSupported(sourceType)) {
				throw new Error('"' + sourceType + '" is not supported.');
			}

			var triggerPush = function triggerPush(_) {

				if (typeof push == 'function') {
					push(_);
				}

				return _;
			};

			var outputTypes = Array.isArray(config.format) ? config.format : [config.format],
			    widths = Array.isArray(config.width) ? config.width : [config.width];

			if (!outputTypes.length) {
				outputTypes.push(sourceType);
			}

			if (!widths.length) {
				widths.push(1);
			}

			var skipOptimization = config.skipOptimization,
			    onlyOptimize = Extensions.svg.test(sourceType) || Extensions.gif.test(sourceType),
			    processors = [];


			outputTypes.forEach(function (type) {

				if (!TypeIsSupported(type)) {
					throw new Error('"' + type + '" is not supported.');
				}

				if (onlyOptimize) {

					if (skipOptimization) {
						triggerPush(source);
					} else {
						processors.push(_this._optimizeImage(source, config).then(triggerPush));
					}
				} else {
					widths.forEach(function (width) {

						if (typeof width != 'number') {
							throw new Error('Invalid width parameter.');
						}

						var proc = _this._attachMetadata(source).then(function (image) {
							return _this._processImage(image, type, width, config);
						});

						if (!skipOptimization) {
							proc = proc.then(function (image) {
								return _this._optimizeImage(image, config);
							});
						}

						proc = proc.then(triggerPush);

						processors.push(proc);
					});
				}
			});

			return Promise.all(processors);
		}

		/**
   * Match image file by path and size
   * @param  {Vinyl}                  source
   * @param  {Array<String|Function>} matchers
   * @return {Promise<Boolean>}
   */

	}, {
		key: 'matchImage',
		value: function matchImage(source) {
			var matcherOrMatchers = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;


			if (!_vinyl2.default.isVinyl(source) || source.isNull() || source.isStream()) {
				throw new Error('Invalid source.');
			}

			var TypeIsSupported = SrcsetGenerator.TypeIsSupported,
			    sourceType = source.extname.replace(/^\./, '');


			if (!TypeIsSupported(sourceType)) {
				return Promise.resolve(false);
			}

			if (!matcherOrMatchers) {
				return Promise.resolve(true);
			}

			var matchers = Array.isArray(matcherOrMatchers) ? matcherOrMatchers : [matcherOrMatchers];

			return this._attachMetadata(source).then(function () {
				var metadata = source.metadata,
				    path = source.path;


				if ((typeof metadata === 'undefined' ? 'undefined' : _typeof(metadata)) != 'object') {
					return false;
				}

				var size = {
					width: metadata.width,
					height: metadata.height
				};

				return matchers.every(function (funcOrPatternOrMediaQuery) {

					if (isMediaQuery.test(funcOrPatternOrMediaQuery)) {
						return _cssMediaquery2.default.match(funcOrPatternOrMediaQuery, size);
					} else if (typeof funcOrPatternOrMediaQuery == 'function') {
						return funcOrPatternOrMediaQuery(path, size, source);
					} else {
						return (0, _minimatch2.default)(path, funcOrPatternOrMediaQuery);
					}
				});
			});
		}

		/**
   * Attach image metadata to the vinyl file.
   * @param  {Vinyl} source
   * @return {Vinyl}
   */

	}, {
		key: '_attachMetadata',
		value: function _attachMetadata(source) {

			if (_typeof(source.metadata) == 'object') {
				return Promise.resolve(source);
			}

			return (0, _sharp2.default)(source.contents).metadata().then(function (metadata) {
				source.metadata = metadata;
				return source;
			});
		}

		/**
   * Resize and convert image.
   * @param  {Vinyl}   source
   * @param  {String}  outputType
   * @param  {Boolean} width
   * @param  {Object}  config
   * @return {Promise<Vinyl>}
   */

	}, {
		key: '_processImage',
		value: function _processImage(source, outputType) {
			var width = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
			var config = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
			var Extensions = SrcsetGenerator.Extensions,
			    metadata = source.metadata,
			    originWidth = (typeof metadata === 'undefined' ? 'undefined' : _typeof(metadata)) == 'object' ? metadata.width : 0,
			    processing = Object.assign({}, this.processing, config.processing),
			    target = source.clone({ contents: false }),
			    processor = (0, _sharp2.default)(source.contents);


			target.extname = '.' + outputType;

			if (width) {

				var calculatedWidth = originWidth && width <= 1 ? width * originWidth : width;

				this._addPostfix(target, calculatedWidth, width, config.postfix);

				if (calculatedWidth < originWidth) {
					processor.resize(calculatedWidth);
				}
			} else {
				this._addPostfix(target, originWidth, originWidth, config.postfix);
			}

			if (width == 1 && source.extname == target.extname) {
				target.contents = source.contents;
				return Promise.resolve(target);
			}

			if (Extensions.webp.test(outputType)) {
				processor.webp(processing.webp);
			} else if (Extensions.jpg.test(outputType)) {
				processor.jpeg(processing.jpg);
			} else if (Extensions.png.test(outputType)) {
				processor.png(processing.png);
			}

			return processor.toBuffer().then(function (buffer) {
				target.contents = buffer;
				return target;
			});
		}

		/**
   * Optimize image with imagemin.
   * @param  {Vinyl}  source
   * @param  {Object} config
   * @return {Promise<Vinyl>}
   */

	}, {
		key: '_optimizeImage',
		value: function _optimizeImage(source) {
			var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};


			var target = source.clone({ contents: false }),
			    optimization = Object.assign({}, this.optimization, config.optimization);

			return _imagemin2.default.buffer(source.contents, {
				plugins: [optimization[source.extname.replace(/^\./, '')]]
			}).then(function (buffer) {
				target.contents = buffer;
				return target;
			});
		}

		/**
   * Add postfix to image file name.
   * @param {Vinyl}           target
   * @param {Number}          calculatedWidth
   * @param {Number}          width
   * @param {String|Function} customPostfix
   */

	}, {
		key: '_addPostfix',
		value: function _addPostfix(target, calculatedWidth, width) {
			var customPostfix = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
			var postfix = this.postfix;


			if (typeof customPostfix == 'string') {
				target.stem += customPostfix;
			} else if (typeof customPostfix == 'function') {
				target.stem += customPostfix(calculatedWidth, width);
			} else if (typeof postfix == 'function') {
				target.stem += postfix(calculatedWidth, width);
			}
		}
	}]);

	return SrcsetGenerator;
}();

SrcsetGenerator.Extensions = extensions;
SrcsetGenerator.ExtensionsList = Object.keys(extensions).map(function (_) {
	return extensions[_];
});
exports.default = SrcsetGenerator;
module.exports = exports['default'];